/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#ifndef TArmeAudioFileHandler_h
#include "TArmeAudioFileHandler.h"
#endif //TArmeAudioFileHandler_h

#ifndef TArmeMediaEngine_h
#include "TArmeMediaEngine.h"
#endif //TArmeMediaEngine_h

#ifndef TArmeDoc_h
#include "TArmeDoc.h"
#endif //TArmeDoc_h

#ifndef Zz_PathHelpers_h
#include "Zz_PathHelpers.h"
#endif //Zz_PathHelpers_h

#ifndef MEAudioClipData_h
#include "MEAudioClipData.h"
#endif //MEAudioClipData_h

#ifndef MEAudioClip_h
#include "MEAudioClip.h"
#endif //MEAudioClip_h

#ifndef ZzHelpers_h
#include "ZzHelpers.h"
#endif //ZzHelpers_h

#ifndef MESFPAudioFile_h
#include "MESFPAudioFile.h"
#endif //MESFPAudioFile_h

#ifndef MEApplicationServices_h
#include "MEApplicationServices.h"
#endif //MEApplicationServices_h

SInt32 TArmeAudioFileHandler::GetSupportedTypesCount()
{
	return 0;
}

char* TArmeAudioFileHandler::GetSupportedType(int32 i)
{
	ZzRtAssert(0);
	return NULL;
}

bool TArmeAudioFileHandler::CanHandleFileByLookingAtExtension( const ZzString* path )
{
#ifdef DEBUG
	printf("TArmeAudioFileHandler::CanHandleFileByLookingAtExtension(%s)\n",path->getUTF8String() );
#endif //#ifdef DEBUG
	
	// if external references are not allowed then don't even bother to check. return false
	bool refs = TArmeMediaEngine::ApplicationServices()->IsReferencingExternalFiles();
	if(!refs)
		return false;
	
	// otherwise check if this file can be referenced
	ArmeErr err = MESFPAudioFile::IsFileSuitableForRealTime(*path);
	return (err.type == kArmeOK);
}

bool TArmeAudioFileHandler::CanHandleFileByLookingAtType( const ZzString* path )
{
#ifdef DEBUG
	printf("TArmeAudioFileHandler::CanHandleFileByLookingAtType(%s)\n",path->getUTF8String() );
#endif //#ifdef DEBUG
	// $$$HACK we don't need this. we do the whole job in CanHandleFileByLookingAtExtension 
	// which is much more than simple extension checking. Imho, we need a more abstract way 
	// of checking a handler can handle a file or not
	return FALSE; 
}

ArmeErr TArmeAudioFileHandler::CreateWSObjsFromFile( 
				TArmeDoc* 					pDoc,
				ZzArchPtrArray*				theObjects,
				const ZzString*				cstrPath,
				ArmeDataFileRef*			fileRef, 
				const TArmeWSObjPos&		pos,
				MIProgressBarDlgManager*	progressMngr)
{
#ifdef DEBUG
	printf("TArmeAudioFileHandler::CreateWSObjsFromFile(%s)\n",cstrPath->getUTF8String() );
#endif //#ifdef DEBUG
	ZzString* theFilePath = new ZzString(*cstrPath);

	//if referencing external files, we should create an alias in the project package in order to trace the external file movement
	if (TArmeMediaEngine::ApplicationServices()->IsReferencingExternalFiles())
	{
		TArmeMediaEngine::ApplicationServices()->CreateAliasFileInProjectPackage(theFilePath, cstrPath);
	}

	// allocate a new chunk
	MEAudioChunkFileIndex chunkFileIndex = pDoc->AudioChunkFileManager()->AllocateNewZeroRefMTGAChunkIndex();
	
	// set its path
	pDoc->AudioChunkFileManager()->SetChunkFileExternalSFP(chunkFileIndex,theFilePath->getUTF8String());

	// retrieve file info
	SInt64 fileNbFrames = (SInt64)(MESFPAudioFile::GetFileNbFrames(*cstrPath));  //don't know why I can't use the alias file here
	SInt32 nbChannels = (SInt32)(MESFPAudioFile::GetFileNbChannels(*cstrPath));  //don't know why I can't use the alias file here

	// create a chunk referencing the full file
	MEAudioChunk chunk(pDoc);
	chunk.SetFileIndex(chunkFileIndex);
	chunk.startInFile = 0;
	chunk.duration = fileNbFrames;
	
	// create a clip made of clip data containing a chunk list that contains the single chunk
	MEAudioChunkList chunkList(chunk);
	MEAudioClipData* clipData = new MEAudioClipData(pDoc,chunkList,nbChannels,progressMngr);
	MEAudioClip* clip = new MEAudioClip(pDoc,clipData);
	
	// set the required pos
	clip->SetPos(pos);
	
	char utf8Name[1024];
	ZzGetFileNameFromPath(utf8Name, 1024, cstrPath->getUTF8String());
	clip->SetNameCopy(utf8Name);
		
	// add the created clip to the list of clips
	theObjects->AddItem(clip);
	
	delete theFilePath;
	return ArmeErr(kArmeOK);
}

char* TArmeAudioFileHandler::fsExtensions[] =
{
	"wav ",
	"wav",
	"aif ",
	"aif",
	"aiff",
	"aifc",
	"cdda",
	"sd2 ",
	"sd2",
	// don't forget to modify the GetSupportedExtensionsCount() as you add new extensions here
};

int32 TArmeAudioFileHandler::GetSupportedExtensionsCount()
{
	return 9;
}

char* TArmeAudioFileHandler::GetSupportedExtension(int32 i)
{
	return fsExtensions[i];
}






